@@ -36,6 +36,7 @@ from keystoneauth1 import session
 from keystoneauth1 import exceptions as ks_exceptions
 from keystoneclient import discover
 import keystoneauth1.identity.generic as ks_auth
+from mistralclient.api import base as mistralclient_base
 from mistralclient.api import client as mistralclient
 import novaclient as nc
 from novaclient import client as novaclient
@@ -1482,10 +1483,82 @@ def _migrate_plans(mistral, swift, plans):
             swift.get_object(plan, plan_env_filename)
         except swiftclient.ClientException:
             LOG.info('Migrating environment for plan %s to Swift.' % plan)
-            env = mistral.environments.get(plan).variables
-            yaml_string = yaml.safe_dump(env, default_flow_style=False)
-            swift.put_object(plan, plan_env_filename, yaml_string)
-            mistral.environments.delete(plan)
+            try:
+                env = mistral.environments.get(plan).variables
+            except (mistralclient_base.APIException,
+                    ks_exceptions.http.NotFound):
+                LOG.warning('Could not find plan "%s" environment in Mistral '
+                            '- nothing to migrate.' % plan)
+            else:
+                yaml_string = yaml.safe_dump(env, default_flow_style=False)
+                swift.put_object(plan, plan_env_filename, yaml_string)
+                mistral.environments.delete(plan)
+
+
+def _wait_for_mistral_execution(timeout_at, mistral, execution, message='',
+                                fail_on_error=False):
+    while time.time() < timeout_at:
+        exe = mistral.executions.get(execution.id)
+        if exe.state == "RUNNING":
+            time.sleep(5)
+            continue
+        if exe.state == "SUCCESS":
+            return
+        else:
+            exe_out = ""
+            exe_created_at = time.strptime(exe.created_at,
+                                           "%Y-%m-%d %H:%M:%S")
+            ae_list = mistral.action_executions.list()
+            for ae in ae_list:
+                if ((ae.task_name == "run_validation") and
+                    (ae.state == "ERROR") and
+                    (time.strptime(ae.created_at,  "%Y-%m-%d %H:%M:%S") >
+                     exe_created_at)):
+                    task = mistral.tasks.get(ae.task_execution_id)
+                    task_res = task.to_dict().get('result')
+                    exe_out = "%s %s" % (exe_out, task_res)
+            error_message = "ERROR %s %s Mistral execution ID: %s" % (
+                message, exe_out, execution.id)
+            LOG.error(error_message)
+            if fail_on_error:
+                raise RuntimeError(error_message)
+            return
+    else:
+        exe = mistral.executions.get(execution.id)
+        error_message = ("TIMEOUT waiting for execution %s to finish. "
+                         "State: %s" % (exe.id, exe.state))
+        LOG.error(error_message)
+        if fail_on_error:
+            raise RuntimeError(error_message)
+
+
+def _get_session():
+    user, password, project, auth_url = _get_auth_values()
+    auth_kwargs = {
+        'auth_url': auth_url,
+        'username': user,
+        'password': password,
+        'project_name': project,
+        'project_domain_name': 'Default',
+        'user_domain_name': 'Default',
+    }
+    auth_plugin = ks_auth.Password(**auth_kwargs)
+    return session.Session(auth=auth_plugin)
+
+
+def _run_validation_groups(groups=[], mistral_url='', timeout=360,
+                           fail_on_error=False):
+    sess = _get_session()
+    mistral = mistralclient.client(mistral_url=mistral_url, session=sess)
+    LOG.info('Starting and waiting for validation groups %s ', groups)
+    execution = mistral.executions.create(
+        'tripleo.validations.v1.run_groups',
+        workflow_input={'group_names': groups}
+    )
+    fail_message = ("error running the validation groups %s " % groups)
+    timeout_at = time.time() + timeout
+    _wait_for_mistral_execution(timeout_at, mistral, execution, fail_message,
+                                fail_on_error)
 
 
 def _create_default_plan(mistral, plans, timeout=360):
@@ -1501,40 +1574,33 @@ def _create_default_plan(mistral, plans, timeout=360):
         'tripleo.plan_management.v1.create_default_deployment_plan',
         workflow_input={'container': plan_name, 'queue_name': queue_name}
     )
-
     timeout_at = time.time() + timeout
-
-    while time.time() < timeout_at:
-        exe = mistral.executions.get(execution.id)
-        if exe.state == "RUNNING":
-            time.sleep(5)
-            continue
-        if exe.state == "SUCCESS":
-            return
-        else:
-            raise RuntimeError(
-                "Failed to create the default Deployment Plan. Please check "
-                "the create_default_deployment_plan execution in Mistral with "
-                "`openstack workflow execution list`.")
-    else:
-        exe = mistral.executions.get(execution.id)
-        LOG.error("Timed out waiting for execution %s to finish. State: %s",
-                  exe.id, exe.state)
-        raise RuntimeError(
-            "Timed out creating the default Deployment Plan. Please check "
-            "the create_default_deployment_plan execution in Mistral with "
-            "`openstack workflow execution list`.")
+    fail_message = ("error creating the default Deployment Plan %s "
+                    "Check the create_default_deployment_plan execution "
+                    "in Mistral with openstack workflow execution list " %
+                    plan_name)
+    _wait_for_mistral_execution(timeout_at, mistral, execution, fail_message,
+                                fail_on_error=True)
 
 
 def _prepare_ssh_environment(mistral):
     mistral.executions.create('tripleo.validations.v1.copy_ssh_key')
 
 
+def _create_logging_cron(mistral):
+    LOG.info('Configuring an hourly cron trigger for tripleo-ui logging')
+    mistral.cron_triggers.create(
+        'publish-ui-logs-hourly',
+        'tripleo.plan_management.v1.publish_ui_logs_to_swift',
+        pattern='0 * * * *'
+    )
+
+
 def _post_config_mistral(instack_env, mistral, swift):
     LOG.info('Configuring Mistral workbooks')
 
     for workbook in [w for w in mistral.workbooks.list()
-                     if 'tripleo' in w.name]:
+                     if w.name.startswith('tripleo')]:
         mistral.workbooks.delete(workbook.name)
 
     managed_tag = 'tripleo-common-managed'
@@ -1550,12 +1616,19 @@ def _post_config_mistral(instack_env, mistral, swift):
     # TODO(d0ugal): From Q onwards we should only ever delete workflows with
     # the tripleo-common tag.
     if 'tripleo-common-managed' in workflow_tags:
-        workflows_delete = [w for w in all_workflows if managed_tag in w.tags]
+        workflows_delete = [w.name for w in all_workflows
+                            if managed_tag in w.tags]
     else:
-        workflows_delete = [w for w in all_workflows if 'tripleo' in w.name]
+        workflows_delete = [w.name for w in all_workflows
+                            if w.name.startswith('tripleo')]
 
-    for workflow in workflows_delete:
-        mistral.workflows.delete(workflow.name)
+    # in order to delete workflows they should have no triggers associated
+    for trigger in [t for t in mistral.cron_triggers.list()
+                    if t.workflow_name in workflows_delete]:
+        mistral.cron_triggers.delete(trigger.name)
+
+    for workflow_name in workflows_delete:
+        mistral.workflows.delete(workflow_name)
 
     for workbook in [f for f in os.listdir(PATHS.WORKBOOK_PATH)
                      if os.path.isfile(os.path.join(PATHS.WORKBOOK_PATH, f))]:
@@ -1567,6 +1640,7 @@ def _post_config_mistral(instack_env, mistral, swift):
     _create_mistral_config_environment(instack_env, mistral)
     _migrate_plans(mistral, swift, plans)
     _create_default_plan(mistral, plans)
+    _create_logging_cron(mistral)
 
     if CONF.enable_validations:
         _prepare_ssh_environment(mistral)
@@ -1575,16 +1649,7 @@ def _post_config_mistral(instack_env, mistral, swift):
 def _post_config(instack_env):
     _copy_stackrc()
     user, password, project, auth_url = _get_auth_values()
-    auth_kwargs = {
-        'auth_url': auth_url,
-        'username': user,
-        'password': password,
-        'project_name': project,
-        'project_domain_name': 'Default',
-        'user_domain_name': 'Default',
-    }
-    auth_plugin = ks_auth.Password(**auth_kwargs)
-    sess = session.Session(auth=auth_plugin)
+    sess = _get_session()
     # TODO(andreykurilin): remove this check with support of novaclient 6.0.0
     if nc.__version__[0] == "6":
         nova = novaclient.Client(2, user, password, project, auth_url=auth_url)
@@ -1702,6 +1767,9 @@ def install(instack_root, upgrade=False):
         _run_orc(instack_env)
         _post_config(instack_env)
         _run_command(['sudo', 'rm', '-f', '/tmp/svc-map-services'], None, 'rm')
+        if upgrade and CONF.enable_validations:  # Run post-upgrade validations
+            mistral_url = instack_env['UNDERCLOUD_ENDPOINT_MISTRAL_PUBLIC']
+            _run_validation_groups(["post-upgrade"], mistral_url)
     except Exception as e:
         LOG.debug("An exception occurred", exc_info=True)
         LOG.error(FAILURE_MESSAGE,
